function resultsprony = run_prony_2D(imf,sizepatch,K)
% (2015)
% 
% Author :
% Jeremy Schmitt (jeremy.schmitt@ens-lyon.fr) 
% --> Contact author Nelly Pustelnik (nelly.pustelnik@ens-lyon.fr)
% 
% Contributors :
% Nelly Pustelnik (nelly.pustelnik@ens-lyon.fr)
% Patrick Flandrin (patrick.flandrin@ens-lyon.fr)
% Pierre Borgnat (pierre.borgnat@ens-lyon.fr)
% Laurent Condat (Laurent.Condat@gipsa-lab.grenoble-inp.fr)
% 
% This software contains some image processing algorithms whose purpose is to be
% used primarily for research.
% 
% This software is governed by the CeCILL license under French law and
% abiding by the rules of distribution of free software.  You can  use,
% modify and/ or redistribute the software under the terms of the CeCILL
% license as circulated by CEA, CNRS and INRIA at the following URL
% "http://www.cecill.info".
% 
% As a counterpart to the access to the source code and  rights to copy,
% modify and redistribute granted by the license, users are provided only
% with a limited warranty  and the software's author,  the holder of the
% economic rights,  and the successive licensors  have only  limited
% liability.
% 
% In this respect, the user's attention is drawn to the risks associated
% with loading,  using,  modifying and/or developing or reproducing the
% software by the user in light of its specific status of free software,
% that may mean  that it is complicated to manipulate,  and  that  also
% therefore means  that it is reserved for developers  and  experienced
% professionals having in-depth computer knowledge. Users are therefore
% encouraged to load and test the software's suitability as regards their
% requirements in conditions enabling the security of their systems and/or
% data to be ensured and,  more generally, to use and operate it in the
% same conditions as regards security.
% 
% The fact that you are presently reading this means that you have had
% knowledge of the CeCILL license and that you accept its terms.
%
% The function prony_2D performs a local Prony spectral analysis on each IMF.
% The spectral analysis is performed according to the work 
% "2D Prony-Huang Transform: A New Tool for 2-D Spectral Analysis,
% J. Schmitt, N. Pustelnik, P. Borgnat, P. Fladrin, L.Condat,
% IEEE Transactions on Image Processing, 2014".
%
%%%%%%%%%INPUTS%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% imf                       : cell of size K containing the IMFs obtained with EMD 
% sizepatch                 : size of patches for each IMF
% K                         : number of IMFs
%
%%%%%%%%%OUTPUTS%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% resultsprony              : results of prony spectral analysis
% resultsprony.amplitude    : amplitude images of
%                             each IMF
% resultsprony.freq         : frequency images of each IMF
% resultsprony.orientation  : orientation images of each IMF
% resultsprony.coherency    : coherency images of each IMF
% resultsprony.model             : denoised IMFs
% resultsprony.crit              : convergence criterion of each IMF
% resultsprony.dc1               : distance to the set of block-Toeplitz matrix
%                             for each IMF
% resultsprony.dc2               : distance to the set of rank 2 matrix
%                             for each IMF
%
%%%%%%%%SUBFONCTIONS%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% prony_2D : compute local prony spectral analysis on a single IMF
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('Prony Spectral analysis');

for k=1:K
    disp (['IMF ',num2str(k)]);
    [amplitude,freq,orientation,coherency,model,crit,dc1,dc2]=prony_2D(imf(:,:,k),sizepatch(k));
    resultsprony.amplitude(:,:,k)   = amplitude;
    resultsprony.freq(:,:,k)        = freq;
    resultsprony.orientation(:,:,k) = orientation;
    resultsprony.coherency(:,:,k)   = coherency;
    resultsprony.model(:,:,k)       = model;
    resultsprony.crit(:,:,k)        = crit;
    resultsprony.dc1(:,:,k)         = dc1;
    resultsprony.dc2(:,:,k)         = dc2;
end

end