function results = run_prox_emd_2D(data,lambda)
% (2015)
% 
% Author :
% Jeremy Schmitt (jeremy.schmitt@ens-lyon.fr) 
% --> Contact author Nelly Pustelnik (nelly.pustelnik@ens-lyon.fr)
% 
% Contributors :
% Nelly Pustelnik (nelly.pustelnik@ens-lyon.fr)
% Patrick Flandrin (patrick.flandrin@ens-lyon.fr)
% Pierre Borgnat (pierre.borgnat@ens-lyon.fr)
% Laurent Condat (Laurent.Condat@gipsa-lab.grenoble-inp.fr)
% 
% This software contains some image processing algorithms whose purpose is to be
% used primarily for research.
% 
% This software is governed by the CeCILL license under French law and
% abiding by the rules of distribution of free software.  You can  use,
% modify and/ or redistribute the software under the terms of the CeCILL
% license as circulated by CEA, CNRS and INRIA at the following URL
% "http://www.cecill.info".
% 
% As a counterpart to the access to the source code and  rights to copy,
% modify and redistribute granted by the license, users are provided only
% with a limited warranty  and the software's author,  the holder of the
% economic rights,  and the successive licensors  have only  limited
% liability.
% 
% In this respect, the user's attention is drawn to the risks associated
% with loading,  using,  modifying and/or developing or reproducing the
% software by the user in light of its specific status of free software,
% that may mean  that it is complicated to manipulate,  and  that  also
% therefore means  that it is reserved for developers  and  experienced
% professionals having in-depth computer knowledge. Users are therefore
% encouraged to load and test the software's suitability as regards their
% requirements in conditions enabling the security of their systems and/or
% data to be ensured and,  more generally, to use and operate it in the
% same conditions as regards security.
% 
% The fact that you are presently reading this means that you have had
% knowledge of the CeCILL license and that you accept its terms.
%
% The function run_prox_emd_2D provides a 2D mode decomposition of the
% signal data.x of size (data.n,data.m). The decomposition is performed
% according to the work "2D Prony-Huang Transform: A New Tool for 2-D
% Spectral Analysis, J. Schmitt, N. Pustelnik, P. Borgnat, P. Fladrin, L.
% Condat, IEEE Transactions on Image Processing, 2014".
%
% The algorithm decomposes the data x into a trend component a (results.tend)
% and an IMF component d (results.imf).
% It consists to solve
% For k=1 : K-1
%     (a_k,d_k) = \arg\min_{a,d} ||x-a-d||_2^2 + lambda_geometry.||A d||_{2,1} + lambda.texture.||D d ||_1
%     x = a_k
% End
% The parameters involved in the criterion can be tuned in order to provide
% good estimates.
%
%%%%%%%%%%%%INPUTS%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% data                  : data to decompose
% data.x                : image
% data.n, data.m        : size of image
%
% lambda                 : parameters of algorithm
% lambda.K               : number of IMFs
% lambda.lambda_geometry : cell of size K containing the regularization
%                          parameters for each trend
% lambda.lambda_texture(_l, _c, _d1, d2) : cell of size K containing the 
%                          regularization parameters for each IMF
%
%%%%%%%%%%%%OUTPUTS%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% results                : results of algorithm
% results.trend          : trend of order K
% results.imf            : 3D matrix (n x m x K) containinh containing the K IMFs
% results.crit           : 2D matrix (niter x K) containing the variation of the
%                          convergence criterion 
%                          ||x-a-d||_2^2 + lambda_geometry.||A d||_{2,1} + lambda.texture.||D d ||_1
%
%%%%%%%%%%%%SUBFUNCTIONS%%%%%%%%%%%%%%%%%%%%%%%%
%
% define_method_emd_2D  : for a single mode, define parameters, functions, operators used in the iterative algorithm 
% 
% prox_emd_2D : extracts a single mode
disp('Mode decomposition')

for k=1:lambda.K
    disp (['IMF ',num2str(k)]);
    [param,dec,func,grad_prox] = define_method_emd_2D(data,lambda,k);
    resimf = prox_emd_2D(data, dec, func, grad_prox, param, k);   
    results.imf(:,:,k) = resimf.imf;
    results.crit(:,k) = resimf.crit;
    data.x = resimf.trend;
end
results.trend = resimf.trend;

end